@@ -34,7 +34,7 @@ void do_deletion(undo_type action)
 		int charlen = char_length(openfile->current->data + openfile->current_x);
 		size_t line_len = strlen(openfile->current->data + openfile->current_x);
 #ifndef NANO_TINY
-		size_t old_amount = openfile->current->extrarows;
+		size_t old_amount = ISSET(SOFTWRAP) ? extra_chunks_in(openfile->current) : 0;
 
 		/* If the type of action changed or the cursor moved to a different
 		 * line, create a new undo item, otherwise update the existing item. */
@@ -49,13 +49,9 @@ void do_deletion(undo_type action)
 					&openfile->current->data[openfile->current_x + charlen],
 					line_len - charlen + 1);
 #ifndef NANO_TINY
-		/* When softwrapping, recompute the number of chunks in the line,
-		 * and schedule a refresh if the number changed. */
-		if (ISSET(SOFTWRAP)) {
-			openfile->current->extrarows = extra_chunks_in(openfile->current);
-			if (openfile->current->extrarows != old_amount)
-				refresh_needed = TRUE;
-		}
+		/* When softwrapping, a changed number of chunks requires a refresh. */
+		if (ISSET(SOFTWRAP) && extra_chunks_in(openfile->current) != old_amount)
+			refresh_needed = TRUE;
 
 		/* Adjust the mark if it is after the cursor on the current line. */
 		if (openfile->mark == openfile->current &&
@@ -94,10 +90,6 @@ void do_deletion(undo_type action)
 
 		unlink_node(joining);
 
-#ifndef NANO_TINY
-		if (ISSET(SOFTWRAP))
-			openfile->current->extrarows = extra_chunks_in(openfile->current);
-#endif
 		/* Two lines were joined, so do a renumbering and refresh the screen. */
 		renumber_from(openfile->current);
 		refresh_needed = TRUE;
@@ -257,27 +249,29 @@ void extract_segment(linestruct *top, size_t top_x, linestruct *bot, size_t bot_
 	bool same_line = (openfile->mark == top);
 	bool post_marked = (openfile->mark && (openfile->mark->lineno > top->lineno ||
 						(same_line && openfile->mark_x > top_x)));
-	bool was_anchored = top->has_anchor;
+	static bool inherited_anchor = FALSE;
+	bool had_anchor = top->has_anchor;
 
 	if (top == bot && top_x == bot_x)
 		return;
 
 	if (top != bot)
 		for (linestruct *line = top->next; line != bot->next; line = line->next)
-			was_anchored |= line->has_anchor;
+			had_anchor |= line->has_anchor;
 #endif
 
 	if (top == bot) {
 		taken = make_new_node(NULL);
 		taken->data = measured_copy(top->data + top_x, bot_x - top_x);
-		memmove(top->data + top_x, top->data + bot_x,
-										strlen(top->data + bot_x) + 1);
+		memmove(top->data + top_x, top->data + bot_x, strlen(top->data + bot_x) + 1);
 		last = taken;
 	} else if (top_x == 0 && bot_x == 0) {
 		taken = top;
 		last = make_new_node(NULL);
 		last->data = copy_of("");
-
+#ifndef NANO_TINY
+		last->has_anchor = bot->has_anchor;
+#endif
 		last->prev = bot->prev;
 		bot->prev->next = last;
 		last->next = NULL;
@@ -317,11 +311,17 @@ void extract_segment(linestruct *top, size_t top_x, linestruct *bot, size_t bot_
 	if (cutbuffer == NULL) {
 		cutbuffer = taken;
 		cutbottom = last;
+#ifndef NANO_TINY
+		inherited_anchor = taken->has_anchor;
+#endif
 	} else {
 		cutbottom->data = nrealloc(cutbottom->data,
 							strlen(cutbottom->data) + strlen(taken->data) + 1);
 		strcat(cutbottom->data, taken->data);
-
+#ifndef NANO_TINY
+		cutbottom->has_anchor = taken->has_anchor && !inherited_anchor;
+		inherited_anchor |= taken->has_anchor;
+#endif
 		cutbottom->next = taken->next;
 		delete_node(taken);
 
@@ -334,10 +334,7 @@ void extract_segment(linestruct *top, size_t top_x, linestruct *bot, size_t bot_
 	openfile->current_x = top_x;
 
 #ifndef NANO_TINY
-	openfile->current->has_anchor = was_anchored;
-
-	if (ISSET(SOFTWRAP))
-		openfile->current->extrarows = extra_chunks_in(openfile->current);
+	openfile->current->has_anchor = had_anchor;
 
 	if (post_marked || same_line)
 		openfile->mark = openfile->current;
@@ -391,10 +388,6 @@ void ingraft_buffer(linestruct *topline)
 	}
 
 	if (topline != botline) {
-#ifndef NANO_TINY
-		/* First compute the softwrapped chunks for each line in the graft. */
-		compute_the_extra_rows_per_line_from(topline);
-#endif
 		/* When inserting at end-of-buffer, update the relevant pointer. */
 		if (line->next == NULL)
 			openfile->filebot = botline;
@@ -427,11 +420,6 @@ void ingraft_buffer(linestruct *topline)
 		openfile->mark_x += length - xpos;
 	} else if (mark_follows)
 		openfile->mark_x += extralen;
-
-	if (ISSET(SOFTWRAP)) {
-		line->extrarows = extra_chunks_in(line);
-		openfile->current->extrarows = extra_chunks_in(openfile->current);
-	}
 #endif
 
 	delete_node(topline);
@@ -706,10 +694,13 @@ void copy_text(void)
 /* Copy text from the cutbuffer into the current buffer. */
 void paste_text(void)
 {
+#ifndef NANO_TINY
+	/* Remember where the paste started. */
+	linestruct *was_current = openfile->current;
+	bool had_anchor = was_current->has_anchor;
+#endif
 	ssize_t was_lineno = openfile->current->lineno;
-		/* The line number where we started the paste. */
 	size_t was_leftedge = 0;
-		/* The leftedge where we started the paste. */
 
 	if (cutbuffer == NULL) {
 		statusline(AHEM, _("Cutbuffer is empty"));
@@ -728,6 +719,12 @@ void paste_text(void)
 	copy_from_buffer(cutbuffer);
 
 #ifndef NANO_TINY
+	/* Wipe any anchors in the pasted text, so that they don't proliferate. */
+	for (linestruct *line = was_current; line != openfile->current->next; line = line->next)
+		line->has_anchor = FALSE;
+
+	was_current->has_anchor = had_anchor;
+
 	update_undo(PASTE);
 #endif
 
