@@ -408,11 +408,13 @@ void draw_the_promptbar(void)
 
 	wattroff(bottomwin, interface_color_pair[PROMPT_BAR]);
 
+#if defined(NCURSES_VERSION_PATCH) && (NCURSES_VERSION_PATCH < 20210220)
 	/* Work around a cursor-misplacement bug -- https://sv.gnu.org/bugs/?59808. */
 	if (ISSET(NO_HELP)) {
 		wmove(bottomwin, 0, 0);
 		wrefresh(bottomwin);
 	}
+#endif
 
 	/* Place the cursor at the right spot. */
 	wmove(bottomwin, 0, column - the_page);
@@ -424,7 +426,7 @@ void draw_the_promptbar(void)
 void add_or_remove_pipe_symbol_from_answer(void)
 {
 	if (answer[0] == '|') {
-		memmove(answer, answer + 1, strlen(answer) + 1);
+		memmove(answer, answer + 1, strlen(answer));
 		if (typing_x > 0)
 			typing_x--;
 	} else {
@@ -764,7 +766,7 @@ int do_yesno_prompt(bool all, const char *msg)
 					choice = -2;
 			}
 		}
-#endif /* ENABLE_MOUSE */
+#endif
 		else if (func_from_key(&kbinput) == full_refresh)
 			full_refresh();
 #ifndef NANO_TINY
