@@ -82,7 +82,7 @@ void search_init(bool replacing, bool retain_answer)
 		thedefault = nmalloc(strlen(disp) + 7);
 		/* We use (COLS / 3) here because we need to see more on the line. */
 		sprintf(thedefault, " [%s%s]", disp,
-				(breadth(last_search) > COLS / 3) ? "..." : "");
+					(breadth(last_search) > COLS / 3) ? "..." : "");
 		free(disp);
 	} else
 		thedefault = copy_of("");
@@ -323,21 +323,18 @@ int findnextstr(const char *needle, bool whole_word_only, int modus,
 		*match_len = found_len;
 
 #ifndef NANO_TINY
-	if (modus == JUSTFIND && ISSET(MARK_MATCH) && (!openfile->mark || openfile->softmark)) {
-		openfile->mark = line;
-		openfile->mark_x = found_x + found_len;
-		openfile->softmark = TRUE;
+	if (modus == JUSTFIND && (!openfile->mark || openfile->softmark)) {
+		spotlighted = TRUE;
+		light_from_col = xplustabs();
+		light_to_col = wideness(line->data, found_x + found_len);
 		if (!ISSET(SHOW_CURSOR))
 			hide_cursor = TRUE;
-		shift_held = TRUE;
+		edit_refresh();
 	}
 #endif
 
-	/* Wipe the "Searching..." message and unsuppress cursor-position display. */
-	if (feedback > 0) {
+	if (feedback > 0)
 		wipe_statusbar();
-		lastmessage = VACUUM;
-	}
 
 	return 1;
 }
@@ -431,10 +428,8 @@ void go_looking(void)
 	/* If we found something, and we're back at the exact same spot
 	 * where we started searching, then this is the only occurrence. */
 	if (didfind == 1 && openfile->current == was_current &&
-				openfile->current_x == was_current_x)
+						openfile->current_x == was_current_x)
 		statusline(REMARK, _("This is the only occurrence"));
-	else if (didfind == 1 && LINES == 1)
-		refresh_needed = TRUE;
 	else if (didfind == 0)
 		not_found_msg(last_search);
 
@@ -475,8 +470,7 @@ int replace_regexp(char *string, bool create)
 			/* And if create is TRUE, append the result of the
 			 * subexpression match to the new line. */
 			if (create) {
-				strncpy(string, openfile->current->data +
-										regmatches[num].rm_so, i);
+				strncpy(string, openfile->current->data + regmatches[num].rm_so, i);
 				string += i;
 			}
 		}
@@ -521,19 +515,6 @@ char *replace_line(const char *needle)
 	return copy;
 }
 
-#ifdef ENABLE_COLOR
-/* Reset the multiline coloring info and then recalculate it. */
-void wipe_and_recalculate_colorinfo(void)
-{
-	for (linestruct *line = openfile->filetop; line != NULL; line = line->next)
-		if (line->multidata)
-			for (short index = 0; index < openfile->syntax->nmultis; index++)
-				line->multidata[index] = -1;
-
-	precalc_multicolorinfo();
-}
-#endif
-
 /* Step through each occurrence of the search string and prompt the user
  * before replacing it.  We seek for needle, and replace it with answer.
  * The parameters real_current and real_current_x are needed in order to
@@ -569,7 +550,7 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 			openfile->current_x = bot_x;
 		}
 	}
-#endif /* !NANO_TINY */
+#endif
 
 	came_full_circle = FALSE;
 
@@ -625,17 +606,16 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 		}
 
 		if (choice == 1 || replaceall) {  /* Yes, replace it. */
-			char *copy;
 			size_t length_change;
+			char *altered;
+
+			altered = replace_line(needle);
+
+			length_change = strlen(altered) - strlen(openfile->current->data);
 
 #ifndef NANO_TINY
 			add_undo(REPLACE, NULL);
-#endif
-			copy = replace_line(needle);
 
-			length_change = strlen(copy) - strlen(openfile->current->data);
-
-#ifndef NANO_TINY
 			/* If the mark was on and it was located after the cursor,
 			 * then adjust its x position for any text length changes. */
 			if (was_mark && !right_side_up) {
@@ -651,8 +631,9 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 
 			/* If the mark was not on or it was before the cursor, then
 			 * adjust the cursor's x position for any text length changes. */
-			if (!was_mark || right_side_up) {
+			if (!was_mark || right_side_up)
 #endif
+			{
 				if (openfile->current == real_current &&
 						openfile->current_x < *real_current_x) {
 					if (*real_current_x < openfile->current_x + match_len)
@@ -660,8 +641,8 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 					*real_current_x += length_change;
 #ifndef NANO_TINY
 					bot_x = *real_current_x;
-				}
 #endif
+				}
 			}
 
 			/* Don't find the same zero-length or BOL match again. */
@@ -674,21 +655,10 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 				openfile->current_x += match_len + length_change;
 
 			/* Update the file size, and put the changed line into place. */
-			openfile->totsize += mbstrlen(copy) - mbstrlen(openfile->current->data);
+			openfile->totsize += mbstrlen(altered) - mbstrlen(openfile->current->data);
 			free(openfile->current->data);
-			openfile->current->data = copy;
+			openfile->current->data = altered;
 
-#ifndef NANO_TINY
-			if (ISSET(SOFTWRAP))
-				openfile->current->extrarows = extra_chunks_in(openfile->current);
-#endif
-#ifdef ENABLE_COLOR
-			/* Check whether the replacement requires a change in the coloring. */
-			check_the_multis(openfile->current);
-
-			if (refresh_needed && !replaceall)
-				wipe_and_recalculate_colorinfo();
-#endif
 			set_modified();
 			as_an_at = TRUE;
 			numreplaced++;
@@ -698,11 +668,6 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 	if (numreplaced == -1)
 		not_found_msg(needle);
 
-#ifdef ENABLE_COLOR
-	if (refresh_needed)
-		wipe_and_recalculate_colorinfo();
-#endif
-
 #ifndef NANO_TINY
 	openfile->mark = was_mark;
 #endif
@@ -735,8 +700,8 @@ void ask_for_and_do_replacements(void)
 	size_t begin_x = openfile->current_x;
 	ssize_t numreplaced;
 	int response = do_prompt(MREPLACEWITH, "", &replace_history,
-						/* TRANSLATORS: This is a prompt. */
-						edit_refresh, _("Replace with"));
+							/* TRANSLATORS: This is a prompt. */
+							edit_refresh, _("Replace with"));
 
 #ifdef ENABLE_HISTORIES
 	/* When not "", add the replace string to the replace history list. */
@@ -758,11 +723,12 @@ void ask_for_and_do_replacements(void)
 	openfile->firstcolumn = was_firstcolumn;
 	openfile->current = beginline;
 	openfile->current_x = begin_x;
-	refresh_needed = TRUE;
+
+	edit_refresh();
 
 	if (numreplaced >= 0)
 		statusline(REMARK, P_("Replaced %zd occurrence",
-				"Replaced %zd occurrences", numreplaced), numreplaced);
+					"Replaced %zd occurrences", numreplaced), numreplaced);
 }
 
 /* Go to the specified line and x position. */
